//=========================================================
//  MusE
//  Linux Music Editor
//  $Id: midithreadp.h,v 1.9 2003/04/02 21:31:28 muse Exp $
//
//  (C) Copyright 2001 Werner Schweer (ws@seh.de)
//=========================================================

enum State {IDLE, PLAY, START_PLAY, SYNC, PRECOUNT};

#include "pos.h"
#include "mpevent.h"

//---------------------------------------------------------
//   MidiThreadPrivate
//---------------------------------------------------------

struct MidiThreadPrivate {
      State state;
      int timerFd;
      int sigFd;

      // play values:
      int playTickPos;        // current midi tick position in song
      int rtcStartTick;
      int endSlice;

      MPEventList playEvents;
      MPEventList stuckNotes;

      bool controlChanged;

      int recTick;            // ext sync tick position
      int lastTickPos;        // position of last sync tick

      // record values:
      int curPitch, curVelo;  // Note Input for step recording etc.
      Pos startRecord;
      Pos endRecord;
      bool loopPassed;

      // run values:
      unsigned midiTick;            // free running midi tick position

      unsigned rtcTick;             // free running rtcTick
      unsigned rtcTickStart;        // pos 0.0.0

      unsigned samplePos;           // audio sample time base
      unsigned samplePosStart;      // pos 0.0.0

      unsigned midiClock;           // next midi clock pos in midi ticks
      int realRtcTicks;             // current rtc tick rate 1/sec
      int tempoSN;                  // last tempo serial number to track
                                    //    tempo changes
      double mclock1, mclock2;

      unsigned midiClick;
      int clickno;
      int clicksMeasure;
      int ticksBeat;

      void startPlay(int);
      void stopPlay();
      void seek(int pos);
      void playEvent(const MidiPlayEvent* event);
      void processTimerTick();
      void processMidiClock();
      void initDevices();
      void realtimeSystemInput(int, int);
      void nextEvents(int starTick, int endTick);
      void panic();
      };

