/*
 * MusE IIWU Synth softsynth plugin
 *
 * Copyright (C) 2002 Robert Ham (node@users.sourcforge.net)
 *
 * $Id: iiwusynthgui.cpp,v 1.17 2003/04/03 22:20:48 muse Exp $
 *
 */

// #include <qt/qapplication.h>
// #include <qt/qlistbox.h>
// #include <qt/qcheckbox.h>
// #include <qt/qslider.h>

#include <iostream>
#include <qtooltip.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qapplication.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qslider.h>
#include <qlineedit.h>
#include <qfiledialog.h>

#include "iiwusynthgui.h"
#include "debug.h"
// #include "filedialog.h"

/*** IIWUSynthGui ***/

IIWUSynthGui::IIWUSynthGui () {
  _bigH = height();
  _smallH = IIWUSynthButton->y() + IIWUSynthButton->height() + 2;
  setFixedSize (width(), _smallH);
  connect(IIWUSynthButton, SIGNAL(toggled (bool)), SLOT(toggleIIWU (bool)));
  connect(Reverb, SIGNAL (toggled (bool)), SLOT(toggleReverb (bool)));
  connect(Chorus, SIGNAL (toggled (bool)), SLOT(toggleChorus (bool)));
  connect(Gain, SIGNAL (valueChanged (int)), SLOT(changeGain (int)));
  connect(ReverbRoomSize, SIGNAL (valueChanged (int)), SLOT(changeReverbRoomSize (int)));
  connect(ReverbDamping, SIGNAL (valueChanged (int)), SLOT(changeReverbDamping (int)));
  connect(ReverbWidth, SIGNAL (valueChanged (int)), SLOT(changeReverbWidth (int)));
  connect(ReverbLevel, SIGNAL (valueChanged (int)), SLOT(changeReverbLevel (int)));
  connect(ChorusNumber, SIGNAL (valueChanged (int)), SLOT(changeChorusNumber (int)));
  connect(ChorusType, SIGNAL (activated (int)), SLOT(changeChorusType (int)));
  connect(ChorusSpeed, SIGNAL (valueChanged (int)), SLOT(changeChorusSpeed (int)));
  connect(ChorusDepth, SIGNAL (valueChanged (int)), SLOT(changeChorusDepth (int)));
  connect(ChorusLevel, SIGNAL (valueChanged (int)), SLOT(changeChorusLevel (int)));

  connect (Push, SIGNAL (clicked ()), SLOT(pushClicked ()));
  connect (File, SIGNAL (clicked ()), SLOT(fileClicked ()));

  _notifier = new QSocketNotifier(0, QSocketNotifier::Read);
  connect(_notifier, SIGNAL(activated(int)), SLOT(readData(int)));

  requestAllParameters ();
}

void IIWUSynthGui::pushClicked () {
  const QString& fns = Filename->text();
  if (fns.isEmpty()) return;
  const char * fn = fns.latin1();

  int datalen = strlen (fn) + 2;
  unsigned char data [datalen];
  data[0] = MUSE_IIWU_SOUNDFONT_PUSH;
  memcpy (data + 1, fn, datalen - 1);
  sendSysex (data, datalen);

  data[0] = MUSE_IIWU_SOUNDFONT_GET;
  sendSysex (data, 1);

  return;
}

void IIWUSynthGui::fileClicked () {
  QString dir = QFileDialog::getOpenFileName(QString::null, QString("*.[Ss][Ff]2"), this);
  Filename->setText (dir);
}

void IIWUSynthGui::readData (int fd) {
  unsigned char buffer[512];
  int n = ::read(fd, buffer, 512);
  dataInput(buffer, n);
}


IIWUSynthGui::~IIWUSynthGui () {
  delete _notifier;
}

void IIWUSynthGui::toggleIIWU (bool on) {
  QToolTip::remove (IIWUSynthButton);
  if (on) {
    setFixedSize (width(), _bigH);
    QToolTip::add (IIWUSynthButton, tr("hide iiwusynth controls"));
  } else {
    setFixedSize (width(), _smallH);
    QToolTip::add (IIWUSynthButton, tr("show iiwusynth controls"));
  }

  int datalen = 1 + sizeof (bool);
  unsigned char data [datalen];
  data[0] = MUSE_IIWU_ADVGUI_SET;
  *((bool *) (data + 1)) = on;
  sendSysex (data, datalen);
}

void IIWUSynthGui::toggleReverb (bool on) {
  ReverbFrame->setEnabled (on);
  sendParameterChange(MUSE_IIWU_PARAMETER_REVERB,
                      "on", ((int) on)*128);
}

void IIWUSynthGui::toggleChorus (bool on) {
  ChorusFrame->setEnabled (on);
  sendParameterChange(MUSE_IIWU_PARAMETER_CHORUS,
                      "on", ((int) on)*128);
}

void IIWUSynthGui::changeGain (int value) {
  int datalen = 1 + sizeof (double);
  unsigned char * data = new unsigned char [datalen];
  *data = MUSE_IIWU_GAIN_SET;
  double * dp = (double *) (data + 1);
  *dp = ( ((double)value) / ((double)128.0)) * ((double)10.0);
  sendSysex (data, datalen);
  delete data;
}

void IIWUSynthGui::changeReverbRoomSize (int value) {
  sendParameterChange(MUSE_IIWU_PARAMETER_REVERB,
                      "roomsize", value);
}

void IIWUSynthGui::changeReverbDamping (int value) {
  sendParameterChange(MUSE_IIWU_PARAMETER_REVERB,
                      "damping", value);
}

void IIWUSynthGui::changeReverbWidth (int value) {
  sendParameterChange(MUSE_IIWU_PARAMETER_REVERB,
                      "width", value);
}

void IIWUSynthGui::changeReverbLevel (int value) {
  sendParameterChange(MUSE_IIWU_PARAMETER_REVERB,
                      "level", value);
}

void IIWUSynthGui::changeChorusNumber (int value) {
  sendParameterChange(MUSE_IIWU_PARAMETER_CHORUS,
                      "number", value);
}

void IIWUSynthGui::changeChorusType (int value) {
  sendParameterChange(MUSE_IIWU_PARAMETER_CHORUS,
                      "type", value);
}

void IIWUSynthGui::changeChorusSpeed (int value) {
  sendParameterChange(MUSE_IIWU_PARAMETER_CHORUS,
                      "speed", value);
}

void IIWUSynthGui::changeChorusDepth (int value) {
  sendParameterChange(MUSE_IIWU_PARAMETER_CHORUS,
                      "depth", value);
}

void IIWUSynthGui::changeChorusLevel (int value) {
  sendParameterChange(MUSE_IIWU_PARAMETER_CHORUS,
                      "level", value);
}


void IIWUSynthGui::sysexReceived(unsigned char const * data, int len) {
  unsigned char * decdata = new unsigned char [len / 2];
  for (int i = 0; i < len / 2; i++) {
    decdata[i] = 0xf0 & (data[i * 2] << 4);
    decdata[i] |= (0xf & (data[(i * 2) + 1]));
  }
  dealWithSysex (decdata, len / 2);
  delete decdata;
}

void IIWUSynthGui::dealWithSysex (unsigned char const * data, int datalen) {
  static bool hadFirstSF = false;
  char * cp;
  double * dp;
  switch (*data) {
    case MUSE_IIWU_PARAMETER_GET:
      cp = (char *) (data + 2);
      dp = (double *) (data + strlen (cp) + 3);
      setParameter ((int) *(data+1), cp, *dp);
      break;
    case MUSE_IIWU_GAIN_GET:
      dp = (double *) (data + 1);
      Gain->setValue ((int) (*dp * 12.8));
      break;
    case MUSE_IIWU_SOUNDFONT_GET: {
      // FIXME: why on earth do two sets of sysex get sent?
      if (!hadFirstSF) {
        hadFirstSF = true;
        break;
      } else {
        hadFirstSF = false;
      }
      cp = (char *) data + 2;
      int count = (int) *(data + 1);
      QStringList sfList;
      while (count) {
        sfList.push_back (QString(cp));
        cp += strlen (cp) + 1;
        count--;
      }
      SoundfontStack->clear();
      SoundfontStack->insertStringList (sfList);
      break;
    }
    case MUSE_IIWU_ADVGUI_GET:
      IIWUSynthButton->setOn (*((bool *) (data + 1)));
      break;
  }
}

void IIWUSynthGui::eventReceived (int a, int b, int c) {
}

void IIWUSynthGui::requestAllParameters () {
  sendParameterRequest (MUSE_IIWU_PARAMETER_REVERB, "on");
  sendParameterRequest (MUSE_IIWU_PARAMETER_REVERB, "roomsize");
  sendParameterRequest (MUSE_IIWU_PARAMETER_REVERB, "damping");
  sendParameterRequest (MUSE_IIWU_PARAMETER_REVERB, "width");
  sendParameterRequest (MUSE_IIWU_PARAMETER_REVERB, "level");
  sendParameterRequest (MUSE_IIWU_PARAMETER_CHORUS, "on");
  sendParameterRequest (MUSE_IIWU_PARAMETER_CHORUS, "number");
  sendParameterRequest (MUSE_IIWU_PARAMETER_CHORUS, "type");
  sendParameterRequest (MUSE_IIWU_PARAMETER_CHORUS, "speed");
  sendParameterRequest (MUSE_IIWU_PARAMETER_CHORUS, "depth");
  sendParameterRequest (MUSE_IIWU_PARAMETER_CHORUS, "level");
  unsigned char data[1];
  data[0] = MUSE_IIWU_GAIN_GET;
  sendSysex (data, 1);
  data[0] = MUSE_IIWU_SOUNDFONT_GET;
  sendSysex (data, 1);
  data[0] = MUSE_IIWU_ADVGUI_GET;
  sendSysex (data, 1);
}

bool IIWUSynthGui::sendParameterRequest (int parameterSet, const char * parameter) {
  size_t parameterMem = strlen (parameter) + 1;
  int datalen = 2 + parameterMem;
  unsigned char * data = new unsigned char [datalen];
  *data = MUSE_IIWU_PARAMETER_GET;
  *(data + 1) = (char) parameterSet;
  memcpy (data + 2, parameter, parameterMem);
  if (!sendSysex (data, datalen)) {
    std::cerr << DEBUG_ARGS << "error sending sysex data" << endl;
    delete data;
    return false;
  }
  delete data;
  return true;
}

void IIWUSynthGui::setParameter (int parameterSet, const char * parameter, double value) {
  int ival = (int) (value * 128);
  std::string ps (parameter);
  if (parameterSet == MUSE_IIWU_PARAMETER_REVERB) {
           if (ps == "roomsize") {
      ReverbRoomSize->setValue (ival);
    } else if (ps == "damping") {
      ReverbDamping->setValue (ival);
    } else if (ps == "width") {
      ReverbWidth->setValue (ival);
    } else if (ps == "level") {
      ReverbLevel->setValue (ival);
    } else if (ps == "on") {
      ReverbFrame->setEnabled (ival);
      Reverb->setChecked (ival);
    }
  } else {
           if (ps == "number") {
      ChorusNumber->setValue (ival);
    } else if (ps == "type") {
      ChorusType->setCurrentItem (ival);
    } else if (ps == "speed") {
      ChorusSpeed->setValue (ival);
    } else if (ps == "depth") {
      ChorusDepth->setValue (ival);
    } else if (ps == "level") {
      ChorusLevel->setValue (ival);
    } else if (ps == "on") {
      ChorusFrame->setEnabled (ival);
      Chorus->setChecked (ival);
    }
  }
}

bool IIWUSynthGui::sendParameterChange (int parameterSet, const char * parameter, int value) {
  size_t parameterMem = strlen (parameter) + 1;
  int datalen = 2 + parameterMem + sizeof (double);
  unsigned char * data = new unsigned char [datalen];
  *data = (unsigned char) MUSE_IIWU_PARAMETER_SET;
  *(data + 1) = (unsigned char) parameterSet;
  memcpy (data + 2, parameter, parameterMem);
  double * dp = (double *) (data + 2 + parameterMem);
  *dp = ((double) value) / ((double) 128.0);
  if (!sendSysex (data, datalen)) {
    std::cerr << DEBUG_ARGS << "error sending sysex data" << endl;
    delete data;
    return false;
  }
  delete data;
  return true;
}

bool IIWUSynthGui::sendSysex (const unsigned char * data, int datalen) {
  putchar (0xf0); // tell MidiRawIn::channelStatus that we're sending sysex

  const unsigned char * p = data;
  int i = datalen;
  while (i > 0) {
    putchar ((*p >> 4) & 0xf);
    putchar (*p & 0xf);
    ++p;
    --i;
  }

  putchar (0xf7); // we're done sending

  fflush (stdout);
  return true;
}


//---------------------------------------------------------
//   main
//---------------------------------------------------------

QString museProject;
QString museGlobalShare;
QString museUser;


int main(int argc, char* argv[])
{
    char * instanceName;
  museUser = getenv("MUSEHOME");
  if (museUser == 0)
    museUser = getenv("HOME");
  museGlobalShare = getenv("MUSE");
  if (museGlobalShare == 0) {
    museGlobalShare = "/usr/muse";
    if (access(museGlobalShare.latin1(), R_OK) != 0) {
      museGlobalShare = "/usr/local/muse";
      if (access(museGlobalShare.latin1(), R_OK) != 0)
        museGlobalShare = museUser;
    }
  }
  instanceName = argv[1];

  QApplication app (argc, argv, true);
  QWidget* w = new IIWUSynthGui ();
  if (argc > 1)
    w->setCaption(QString(instanceName));
  w->show();
  app.connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));
  qApp->exec();
}
