/*
 * MusE IIWU Synth softsynth plugin
 *
 * Copyright (C) 2002 Robert Ham (node@users.sourcforge.net)
 *
 * $Id: iiwusynthgui.h,v 1.12 2002/07/27 21:10:09 bob Exp $
 *
 */

#ifndef __MUSE_IIWUSYNTHGUI_H__
#define __MUSE_IIWUSYNTHGUI_H__

#include <string>
#include <qscrollview.h>
#include <qpixmap.h>
#include <qevent.h>
#include <qpushbutton.h>
#include <qmenubar.h>
#include <qsocketnotifier.h>
#include <alsa/asoundlib.h>

#include "driver/midirawin.h"
#include "debug.h"

#include "iiwusynthguibase.h"

enum {
  MUSE_IIWU_REVERB = 100,
  MUSE_IIWU_REVERB_ROOMSIZE,
  MUSE_IIWU_REVERB_DAMPING,
  MUSE_IIWU_REVERB_WIDTH,
  MUSE_IIWU_REVERB_LEVEL,
  MUSE_IIWU_CHORUS,
  MUSE_IIWU_CHORUS_NUMBER,
  MUSE_IIWU_CHORUS_TYPE,
  MUSE_IIWU_CHORUS_SPEED,
  MUSE_IIWU_CHORUS_DEPTH,
  MUSE_IIWU_CHORUS_LEVEL,
  MUSE_IIWU_GAIN,
  MUSE_IIWU_SOUNDFONT,
  MUSE_IIWU_STRING,
  MUSE_IIWU_STRING_END
};

enum {
  MUSE_IIWU_PARAMETER_SET = 33,
  MUSE_IIWU_PARAMETER_GET,
  MUSE_IIWU_PARAMETER_REVERB,
  MUSE_IIWU_PARAMETER_CHORUS,
  MUSE_IIWU_GAIN_SET,
  MUSE_IIWU_GAIN_GET,
  MUSE_IIWU_SOUNDFONT_PUSH,
  MUSE_IIWU_SOUNDFONT_POP,
  MUSE_IIWU_SOUNDFONT_GET,
  MUSE_IIWU_ADVGUI_SET,
  MUSE_IIWU_ADVGUI_GET,
  MUSE_IIWU_ERROR
};

class IIWUSynthGui : public IIWUSynthGuiBase, public MidiRawIn {

Q_OBJECT

private:
  unsigned _smallH;
  unsigned _bigH;
  QSocketNotifier * _notifier;
  bool sendSysex (const unsigned char *, int);
  bool sendParameterChange (int, const char *, int);
  void setParameter (int, const char *, double);
  void requestAllParameters ();
  bool sendParameterRequest(int, const char *);
  void dealWithSysex (unsigned char const * data, int datalen);
private slots:
  void readData(int);
  void pushClicked ();
  void fileClicked ();
protected:
public:
  virtual void sysexReceived (const unsigned char *, int);
  virtual void eventReceived (int, int, int);
  IIWUSynthGui ();
  ~IIWUSynthGui ();
public slots:
  void toggleIIWU (bool);
  void toggleReverb (bool);
  void toggleChorus (bool);
  void changeGain (int);
  void changeReverbRoomSize (int);
  void changeReverbDamping (int);
  void changeReverbWidth (int);
  void changeReverbLevel (int);
  void changeChorusNumber (int);
  void changeChorusType (int);
  void changeChorusSpeed (int);
  void changeChorusDepth (int);
  void changeChorusLevel (int);
};


#endif /* __MUSE_IIWUSYNTHGUI_H__ */
