/*
 * MusE IIWU Synth softsynth plugin
 *
 * Copyright (C) 2002 Robert Ham (node@users.sourcforge.net)
 *
 * $Id: iiwusynti.cpp,v 1.21 2003/03/10 14:34:02 muse Exp $
 *
 */

#include <list>
#include <string>
#include <iostream>

#include "iiwusynti.h"
#include "iiwusynthgui.h"
#include "debug.h"
#include "midictrl.h"

/*** IIWUParameterSet ***/

IIWUParameterSet::IIWUParameterSet (const IIWUParameterSet& ps)
: _parameters (ps._parameters) {
}


/*** IIWUSynth ***/

IIWUSynth::IIWUSynth (const char * name)
: Mess (name, 2),
  _gain(1.0),
  _gainChanged (false),
  _advancedGui (false) {
  iiwu_settings_t* settings;

  settings = new_iiwu_settings();

//  settings.flags &= ~IIWU_AUDIO;
//  settings.sample_format = IIWU_FLOAT_FORMAT;
  // iiwu_settings_setstr(settings, "audio.driver", "alsa");

  _iiwusynth = new_iiwu_synth (settings);
  if (!_iiwusynth) {
//    M_ERROR("error while creating iiwusynth");
    return;
  }
}

IIWUSynth::~IIWUSynth () {
  int err;
  err = delete_iiwu_synth (_iiwusynth);
  if (err == -1) {
        std::cerr << DEBUG_ARGS << "error while destroying synth: " << iiwu_synth_error(_iiwusynth) << std::endl;
    return;
  }
}

bool IIWUSynth::init () {
  std::cerr << DEBUG_ARGS << "begin" << std::endl;
  std::list<std::string> sf;
//  sf.push_back (std::string("share/soundfonts/test-3.sf2"));
//  sf.push_back (std::string("share/soundfonts/test-2.sf2"));
//  sf.push_back (std::string("share/soundfonts/test-1.sf2"));
//  sf.push_back (std::string("/home/ws/sfont/8MBGMSFX.SF2"));
//  sf.push_back (std::string("/home/node/music/samples/soundfonts/sblive-4mb-gm-gs-mt.sf2"));
//  sf.push_back (std::string("/home/node/music/samples/soundfonts/analogue/vintage-dreams-waves-2.1.sf2"));
//  sf.push_back (std::string("/home/node/music/samples/soundfonts/fluid-r3-gm.sf2"));
  bool berr;
  for (std::list<std::string>::iterator it = sf.begin();
       it != sf.end();
       ++it) {
    berr = pushSoundfont (*it);
    if (!berr) {
        std::cerr << DEBUG_ARGS << "could not load soundfont " << (*it).c_str()
             << ": " << iiwu_synth_error(_iiwusynth) << std::endl;
      return true;
    }
  }
  setReverb ();
  setChorus ();
  std::cerr << DEBUG_ARGS << "end" << std::endl;
  return false;
}

void IIWUSynth::processEvent (MEvent * mev) {
  int err;
  int channel = mev->chan();
  static std::string st;
//  static bool newString = true;
  switch (mev->type()) {
    case SND_SEQ_EVENT_NOTEON:
    case SND_SEQ_EVENT_KEYPRESS:
      err = iiwu_synth_noteon(_iiwusynth, channel,
                              mev->dataA(), mev->dataB());
      if (err) {
        std::cerr << DEBUG_ARGS << "error processing noteon event: " << iiwu_synth_error(_iiwusynth) << std::endl;
      }
      break;
    case SND_SEQ_EVENT_NOTEOFF:
      err = iiwu_synth_noteoff(_iiwusynth, channel,
                               mev->dataA());
      if (err) {
        std::cerr << DEBUG_ARGS << "error processing noteoff event: " << iiwu_synth_error(_iiwusynth) << std::endl;
      }
      break;
    case SND_SEQ_EVENT_PGMCHANGE:
      err = iiwu_synth_program_change(_iiwusynth, channel, mev->dataB());
      if (err) {
        std::cerr << DEBUG_ARGS << "error processing program change event: " << iiwu_synth_error(_iiwusynth) << std::endl;
      }
      break;
    case SND_SEQ_EVENT_CONTROL14:
    case SND_SEQ_EVENT_NONREGPARAM:
    case SND_SEQ_EVENT_REGPARAM:
    case SND_SEQ_EVENT_CONTROLLER:
      static int ctrlLo, ctrlHi, dataLo, dataHi;
      switch(mev->dataA()) {
        case CTRL_LNRPN:  ctrlLo = mev->dataB(); break;
        case CTRL_HNRPN:  ctrlHi = mev->dataB(); break;
        case CTRL_LDATA:  dataLo = mev->dataB(); break;
        case CTRL_HDATA:  dataHi = mev->dataB(); break;
      }
      if (mev->dataA() == CTRL_HDATA) {
        int ctrl = ctrlLo+ctrlHi*128;
        int data = dataLo+dataHi*128;
        err = iiwu_synth_cc(_iiwusynth, channel, ctrl, data);
        if (err) {
          std::cerr << DEBUG_ARGS << "error processing controller event: " << iiwu_synth_error(_iiwusynth) << std::endl;
        }
      }
      break;
    case SND_SEQ_EVENT_PITCHBEND:
      err = iiwu_synth_pitch_bend (_iiwusynth, channel, (mev->dataA()<<7) | mev->dataB());
      if (err) {
        std::cerr << DEBUG_ARGS << "error processing pitch bend event: " << iiwu_synth_error(_iiwusynth) << std::endl;
      }
      break;
    case SND_SEQ_EVENT_SYSEX:
      decodeSysex(mev->data(), mev->dataLen());
      break;
    default:
      std::cerr << DEBUG_ARGS << "unknown MEvent type " << mev->type() << std::endl;
      break;
  }
}


const char * IIWUSynth::getPatchName (int channel,
  int /*hbank*/, int /*lbank*/, int /*prog*/, MType /*type*/) {
  iiwu_preset_t * preset = iiwu_synth_get_channel_preset(_iiwusynth, channel);
  if (!preset) return "no preset";
  return preset->get_name(preset);
}

const MidiPatch * IIWUSynth::getFirstPatch () const {
  static MidiPatch midiPatch;

  midiPatch.typ = 0;
  midiPatch.hbank = 0;

  int sfcount = iiwu_synth_sfcount (_iiwusynth);
  iiwu_preset_t * preset;
  iiwu_sfont_t * sfont;

  for (int sf = 0; sf < sfcount; ++sf) {
    sfont = iiwu_synth_get_sfont(_iiwusynth, sf);

    for (unsigned int bank = 0; bank < 128; ++bank) {
      for (unsigned int patch = 0; patch < 128; ++patch) {
        preset = sfont->get_preset (sfont, bank, patch);
        if (preset) {
          midiPatch.lbank = bank;
          midiPatch.prog = patch;
          midiPatch.name = preset->get_name (preset);
          return &midiPatch;
        }
      }
    }
  }
  return 0;
}

const MidiPatch * IIWUSynth::getNextPatch (int /*channel*/,
  const MidiPatch * patch) const {
  static MidiPatch midiPatch;

  if (!patch) return getFirstPatch ();


  midiPatch.typ = 0;
  midiPatch.hbank = 0;

  int sfcount = iiwu_synth_sfcount (_iiwusynth);
  iiwu_preset_t * preset;
  iiwu_sfont_t * sfont;

  for (int sf = 0; sf < sfcount; ++sf) {
    sfont = iiwu_synth_get_sfont(_iiwusynth, sf);

    for (unsigned int bank = patch->lbank; bank < 128; ++bank) {
      for (unsigned int prog = patch->prog + 1; prog < 128; ++prog) {
        preset = sfont->get_preset (sfont, bank, prog);
        if (preset) {
          midiPatch.lbank = bank;
          midiPatch.prog = prog;
          midiPatch.name = preset->get_name (preset);
          return &midiPatch;
        }
      }
    }
  }

  return 0;
}


void IIWUSynth::write (int n, float ** ports, int offset) {
  int err;
  err = iiwu_synth_write_float(_iiwusynth, n, ports[0], offset, 1,
                                              ports[1], offset, 1);
  if (err) {
      std::cerr << DEBUG_ARGS << "error writing from synth: " << iiwu_synth_error(_iiwusynth) << std::endl;
      return;
  }
}

void IIWUSynth::decodeSysex (const unsigned char * data, int len) {
  data += 1;
  len -= 2;
  unsigned char * decdata = new unsigned char [len / 2];
  for (int i = 0; i < len / 2; i++) {
    decdata[i] = 0xf0 & (data[i * 2] << 4);
    decdata[i] |= (0xf & (data[(i * 2) + 1]));
  }
  processSysex (decdata, len / 2);
  delete decdata;
}

void IIWUSynth::encodeSysex (const unsigned char * data, int datalen) {
  int ndatalen = (datalen*2);
  unsigned char ndata [ndatalen];
  const unsigned char * p = data;


  for (int i = 0; i < datalen; ++p, ++i) {
    ndata[(i*2)] = (*p >> 4) & 0xf;
    ndata[(i*2)+1] = (*p & 0xf);
  }

  sendSysex (ndata, ndatalen);
}

void IIWUSynth::processSysex (const unsigned char * data, int len) {
  char * cp;
  double * dp;
  double d;
  int ndatalen;
  unsigned char * ndata;
  switch (*data) {
    case MUSE_IIWU_PARAMETER_SET:
      cp = ((char *) data) + 2;
      dp = (double *) (cp + strlen (cp) + 1);
      if (*(data+1) == MUSE_IIWU_PARAMETER_REVERB) {
        _reverbParameters.setParameter (std::string(cp), *dp);
        setReverb();
      } else {
        _chorusParameters.setParameter (std::string(cp), *dp);
        setChorus();
      }
      break;
    case MUSE_IIWU_PARAMETER_GET:
      cp = ((char *) data) + 2;
      if (*(data+1) == MUSE_IIWU_PARAMETER_REVERB) {
        d = _reverbParameters.getParameter (std::string(cp));
      } else {
        d = _chorusParameters.getParameter (std::string(cp));
      }
      ndatalen = len + sizeof(double);
      ndata = new unsigned char [ndatalen];
      memcpy (ndata, data, len);
      dp = (double *) (ndata + len);
      *dp = d;
      encodeSysex (ndata, ndatalen);
      delete ndata;
      break;
    case MUSE_IIWU_GAIN_SET:
      dp = (double *) (data + 1);
      setGain (*dp);
      break;
    case MUSE_IIWU_GAIN_GET:
      ndatalen = len + sizeof (double);
      ndata = new unsigned char [ndatalen];
      memcpy (ndata, data, len);
      dp = (double *) (ndata + len);
      *dp = _gain;
      encodeSysex (ndata, ndatalen);
      delete ndata;
      break;
    case MUSE_IIWU_SOUNDFONT_PUSH:
      cp = ((char *) data) + 1;
      if (!pushSoundfont (std::string(cp))) {
        std::string s ("could not load soundfont ");
        s += cp;
        sendSysexError (s.c_str());
      }
      break;
    case MUSE_IIWU_SOUNDFONT_POP:
      break;
    case MUSE_IIWU_SOUNDFONT_GET: {
      ndatalen = 2;
      for (std::list<std::string>::iterator it = _soundfontStack.begin();
           it != _soundfontStack.end();
           ++it) {
        ndatalen += 1 + strlen (it->c_str());
      }
      ndata = new unsigned char [ndatalen];
      *ndata = MUSE_IIWU_SOUNDFONT_GET;
      *(ndata + 1) = (unsigned char) _soundfontStack.size ();
      std::cerr << DEBUG_ARGS << "sf stack size: " << (int) *(ndata + 1) << std::endl;
      cp = (char *) (ndata + 2);
      int cplen;
      for (std::list<std::string>::iterator it = _soundfontStack.begin();
           it != _soundfontStack.end();
           ++it) {
        cplen = strlen (it->c_str()) + 1;
        memcpy (cp, it->c_str(), cplen);
        cp += cplen;
      }
      encodeSysex (ndata, ndatalen);
      break;
    }
    case MUSE_IIWU_ADVGUI_SET:
      _advancedGui = *((bool *) (data + 1));
      break;
    case MUSE_IIWU_ADVGUI_GET: {
      int ndatalen = 1 + sizeof (bool);
      unsigned char ndata [ndatalen];
      ndata[0] = MUSE_IIWU_ADVGUI_GET;
      *((bool *) (ndata + 1)) = _advancedGui;
      encodeSysex (ndata, ndatalen);
      break;
    }
  }
}

void IIWUSynth::sendSysexError (const char * errorMessage) {
  int len = 2 + strlen (errorMessage);
  unsigned char * data = new unsigned char [len];
  *data = MUSE_IIWU_ERROR;
  memcpy (data + 1, errorMessage, len - 1);
  encodeSysex (data, len);
  delete data;
}

bool IIWUSynth::pushSoundfont (const std::string& filename) {
  int err;
  err = iiwu_synth_sfload(_iiwusynth, filename.c_str(), 1);
  if (err) {
      std::cerr << DEBUG_ARGS << "error loading soundfont: " << iiwu_synth_error(_iiwusynth) << std::endl;
      sendSysexError (iiwu_synth_error(_iiwusynth));
      return false;
  }
  _soundfontStack.push_front (filename);
  return true;
}

bool IIWUSynth::popSoundfont () {
  return true;
}

void IIWUSynth::setReverb () {
  int i = (int) _reverbParameters.getParameter (std::string("on"));
  iiwu_synth_set_reverb_on(_iiwusynth, i);
  if (i) {
    iiwu_synth_set_reverb (_iiwusynth,
                           _reverbParameters.getParameter (std::string("roomsize")),
                           _reverbParameters.getParameter (std::string("damping")),
                           _reverbParameters.getParameter (std::string("width")),
                           _reverbParameters.getParameter (std::string("level")));
  }
}

void IIWUSynth::setChorus () {
  int i = (int) _chorusParameters.getParameter (std::string("on"));
  iiwu_synth_set_chorus_on(_iiwusynth, i);
  if (i) {
    iiwu_synth_set_chorus (_iiwusynth,
                           (int) _chorusParameters.getParameter (std::string("number")),
                           _chorusParameters.getParameter (std::string("level")),
                           _chorusParameters.getParameter (std::string("speed")),
                           _chorusParameters.getParameter (std::string("depth")),
                           (int) _chorusParameters.getParameter (std::string("type")));
  }
}

void IIWUSynth::setGain (double gain) {
  _gain = gain;
  _gainChanged = true;
  iiwu_synth_set_gain (_iiwusynth, (float) gain);
}

int IIWUSynth::getMidiInitEvent(int id, RawMidiEvent* ev) {
     return 0;
}

/*** Muse Plugin stuff ***/

static Mess * inst (const char* name) {
  return new IIWUSynth (name);
}

void initMess() {
  Mess::initMess(1225, "iiwusynth",
    "Bob Ham (node@users.sourceforge.net)",
    "iiwusynth soft synth", "sucks", 2, inst);
}
