/*
 * MusE IIWU Synth softsynth plugin
 *
 * Copyright (C) 2002 Robert Ham (node@users.sourcforge.net)
 *
 * $Id: iiwusynti.h,v 1.12 2002/09/13 17:33:37 muse Exp $
 *
 */

#ifndef __MUSE_IIWUSYNTI_H__
#define __MUSE_IIWUSYNTI_H__

#include <string>
#include <list>
#include <map>
#include <iiwusynth.h>

#include "mess.h"
#include "debug.h"


class IIWUParameterSet {
private:
  std::map <std::string, std::pair <bool, double> > _parameters;
public:
  IIWUParameterSet () {}
  IIWUParameterSet (const IIWUParameterSet&);
  void setParameter (std::string param, int val) {
    _parameters[param] = std::pair<bool, double> (true, ((double)val)/128.0);
  }
  void setParameter (std::string param, double val) {
    _parameters[param] = std::pair<bool, double> (true, val);
  }
  double getParameter (std::string param) { return _parameters[param].second; }
  bool getParameterModified (std::string param) { return _parameters[param].first; }
};

class IIWUSynth : public Mess {
private:
  std::list<std::string> _soundfontStack;
  iiwu_synth_t * _iiwusynth;
  double _gain;
  bool _gainChanged;
  IIWUParameterSet _reverbParameters;
  IIWUParameterSet _chorusParameters;
  bool _advancedGui;

  void setReverb ();
  void setChorus ();
  void setGain (double gain);
  void sendSysexError (const char *);
  virtual void write (int, float **, int);
  void decodeSysex (const unsigned char *, int);
  void processSysex (const unsigned char *, int);
  void encodeSysex(const unsigned char *, int);
  const MidiPatch * getFirstPatch () const;
public:
  IIWUSynth (const char *);
  virtual ~IIWUSynth ();
  virtual bool init ();
  virtual void processEvent (MEvent *);
  virtual const char * getPatchName (int, int, int, int, MType);
  virtual const MidiPatch * getNextPatch (int, const MidiPatch *) const;
  virtual int getMidiInitEvent(int id, RawMidiEvent* ev);
  bool pushSoundfont (const std::string&);
  bool popSoundfont ();
};

#endif /* __MUSE_IIWUSYNTI_H__ */
